@@ -16,11 +16,12 @@
 import json
 import os
 import re
+import threading
+import time
 import unittest
 
 import mock
 
-from rally.cmd import envutils
 from tests.functional import utils
 
 
@@ -95,37 +96,37 @@ class TaskTestCase(unittest.TestCase):
 
     def test_results_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task results --uuid %s" % FAKE_TASK_UUID)
 
     def test_abort_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task abort --uuid %s" % FAKE_TASK_UUID)
 
     def test_delete_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task delete --uuid %s" % FAKE_TASK_UUID)
 
     def test_detailed_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task detailed --uuid %s" % FAKE_TASK_UUID)
 
     def test_report_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task report --tasks %s" % FAKE_TASK_UUID)
 
     def test_sla_check_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task sla_check --uuid %s" % FAKE_TASK_UUID)
 
     def test_status_with_wrong_task_id(self):
         rally = utils.Rally()
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task status --uuid %s" % FAKE_TASK_UUID)
 
     def test_report_one_uuid(self):
@@ -136,14 +137,20 @@ class TaskTestCase(unittest.TestCase):
         rally("task report --out %s" % rally.gen_report_path(extension="html"))
         self.assertTrue(os.path.exists(
             rally.gen_report_path(extension="html")))
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
+                          rally, "task report --report %s" % FAKE_TASK_UUID)
+        rally("task report --junit --out %s" %
+              rally.gen_report_path(extension="junit"))
+        self.assertTrue(os.path.exists(
+            rally.gen_report_path(extension="junit")))
+        self.assertRaises(utils.RallyCliError,
                           rally, "task report --report %s" % FAKE_TASK_UUID)
 
     def test_report_bunch_uuids(self):
         rally = utils.Rally()
         cfg = self._get_sample_task_config()
         config = utils.TaskConfig(cfg)
-        task_uuids = list()
+        task_uuids = []
         for i in range(3):
             res = rally("task start --task %s" % config.filename)
             for line in res.splitlines():
@@ -158,7 +165,7 @@ class TaskTestCase(unittest.TestCase):
         rally = utils.Rally()
         cfg = self._get_sample_task_config()
         config = utils.TaskConfig(cfg)
-        files = list()
+        files = []
         for i in range(3):
             rally("task start --task %s" % config.filename)
             path = "/tmp/task_%d.json" % i
@@ -196,7 +203,7 @@ class TaskTestCase(unittest.TestCase):
                                    rally.gen_report_path(extension="html")))
         self.assertTrue(os.path.exists(
             rally.gen_report_path(extension="html")))
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task report --report %s" % FAKE_TASK_UUID)
 
     def test_delete(self):
@@ -228,7 +235,7 @@ class TaskTestCase(unittest.TestCase):
         self.assertIn(
             "deployment_name", rally("task list --all-deployments"))
 
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally, "task list --status not_existing_status")
 
     def test_list_with_print_uuids_option(self):
@@ -241,7 +248,7 @@ class TaskTestCase(unittest.TestCase):
 
         # Validate against a single task
         res = rally("task start --task %s" % config.filename)
-        task_uuids = list()
+        task_uuids = []
         for line in res.splitlines():
             if "finished" in line:
                 task_uuids.append(line.split(" ")[1][:-1])
@@ -270,11 +277,10 @@ class TaskTestCase(unittest.TestCase):
 
     def test_validate_is_invalid(self):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
         cfg = {"invalid": "config"}
         config = utils.TaskConfig(cfg)
-        self.assertRaises(utils.RallyCmdError,
+        self.assertRaises(utils.RallyCliError,
                           rally,
                           ("task validate --task %(task_file)s "
                            "--deployment %(deployment_id)s") %
@@ -283,28 +289,61 @@ class TaskTestCase(unittest.TestCase):
 
     def test_start(self):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            cfg = self._get_sample_task_config()
-            config = utils.TaskConfig(cfg)
-            output = rally(("task start --task %(task_file)s "
-                            "--deployment %(deployment_id)s") %
-                           {"task_file": config.filename,
-                            "deployment_id": deployment_id})
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        cfg = self._get_sample_task_config()
+        config = utils.TaskConfig(cfg)
+        output = rally(("task start --task %(task_file)s "
+                        "--deployment %(deployment_id)s") %
+                       {"task_file": config.filename,
+                        "deployment_id": deployment_id})
         result = re.search(
             r"(?P<task_id>[0-9a-f\-]{36}): started", output)
         self.assertIsNotNone(result)
 
+    def test_validate_with_plugin_paths(self):
+        rally = utils.Rally()
+        plugin_paths = ("tests/functional/extra/fake_dir1/,"
+                        "tests/functional/extra/fake_dir2/")
+        task_file = "tests/functional/extra/test_fake_scenario.json"
+        output = rally(("--plugin-paths %(plugin_paths)s "
+                        "task validate --task %(task_file)s") %
+                       {"task_file": task_file,
+                        "plugin_paths": plugin_paths})
+
+        self.assertIn("Task config is valid", output)
+
+        plugin_paths = ("tests/functional/extra/fake_dir1/"
+                        "fake_plugin1.py,"
+                        "tests/functional/extra/fake_dir2/"
+                        "fake_plugin2.py")
+        task_file = "tests/functional/extra/test_fake_scenario.json"
+        output = rally(("--plugin-paths %(plugin_paths)s "
+                        "task validate --task %(task_file)s") %
+                       {"task_file": task_file,
+                        "plugin_paths": plugin_paths})
+
+        self.assertIn("Task config is valid", output)
+
+        plugin_paths = ("tests/functional/extra/fake_dir1/,"
+                        "tests/functional/extra/fake_dir2/"
+                        "fake_plugin2.py")
+        task_file = "tests/functional/extra/test_fake_scenario.json"
+        output = rally(("--plugin-paths %(plugin_paths)s "
+                        "task validate --task %(task_file)s") %
+                       {"task_file": task_file,
+                        "plugin_paths": plugin_paths})
+
+        self.assertIn("Task config is valid", output)
+
     def _test_start_abort_on_sla_failure_success(self, cfg, times):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            config = utils.TaskConfig(cfg)
-            rally(("task start --task %(task_file)s "
-                   "--deployment %(deployment_id)s --abort-on-sla-failure") %
-                  {"task_file": config.filename,
-                   "deployment_id": deployment_id})
-            results = json.loads(rally("task results"))
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(cfg)
+        rally(("task start --task %(task_file)s "
+               "--deployment %(deployment_id)s --abort-on-sla-failure") %
+              {"task_file": config.filename,
+               "deployment_id": deployment_id})
+        results = json.loads(rally("task results"))
         iterations_completed = len(results[0]["result"])
         self.assertEqual(times, iterations_completed)
 
@@ -372,14 +411,13 @@ class TaskTestCase(unittest.TestCase):
 
     def _test_start_abort_on_sla_failure(self, cfg, times):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            config = utils.TaskConfig(cfg)
-            rally(("task start --task %(task_file)s "
-                   "--deployment %(deployment_id)s --abort-on-sla-failure") %
-                  {"task_file": config.filename,
-                   "deployment_id": deployment_id})
-            results = json.loads(rally("task results"))
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(cfg)
+        rally(("task start --task %(task_file)s "
+               "--deployment %(deployment_id)s --abort-on-sla-failure") %
+              {"task_file": config.filename,
+               "deployment_id": deployment_id})
+        results = json.loads(rally("task results"))
         iterations_completed = len(results[0]["result"])
         self.assertTrue(iterations_completed < times)
 
@@ -507,25 +545,103 @@ class TaskTestCase(unittest.TestCase):
         }
         self._test_start_abort_on_sla_failure(cfg, times)
 
-    # NOTE(oanufriev): Not implemented
+    def _start_task_in_new_thread(self, rally, cfg, report_file):
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(cfg)
+        cmd = (("task start --task %(task_file)s "
+                "--deployment %(deployment_id)s") %
+               {"task_file": config.filename,
+                "deployment_id": deployment_id})
+        report_path = os.path.join(
+            os.environ.get("REPORTS_ROOT", "rally-cli-output-files"),
+            "TaskTestCase", report_file)
+        task = threading.Thread(target=rally, args=(cmd, ),
+                                kwargs={"report_path": report_path})
+        task.start()
+        uuid = None
+        while not uuid:
+            if not uuid:
+                uuid = utils.get_global("RALLY_TASK", rally.env)
+                time.sleep(0.5)
+        return task, uuid
+
     def test_abort(self):
-        pass
+        RUNNER_TIMES = 10
+        cfg = {
+            "Dummy.dummy": [
+                {
+                    "args": {
+                        "sleep": 5
+                    },
+                    "runner": {
+                        "type": "serial",
+                        "times": RUNNER_TIMES
+                    }
+                }
+            ]
+        }
+        rally = utils.Rally()
+        task, uuid = self._start_task_in_new_thread(
+            rally, cfg, "test_abort-thread_with_abort.txt")
+        rally("task abort %s" % uuid)
+        task.join()
+        results = json.loads(rally("task results"))
+        iterations_completed = len(results[0]["result"])
+        # NOTE(msdubov): check that the task is really stopped before
+        #                the specified number of iterations
+        self.assertTrue(iterations_completed < RUNNER_TIMES)
+        self.assertIn("aborted", rally("task status"))
+        report = rally.gen_report_path(extension="html")
+        rally("task report --out %s" % report)
+
+    def test_abort_soft(self):
+        cfg = {
+            "Dummy.dummy": [
+                {
+                    "args": {
+                        "sleep": 2
+                    },
+                    "runner": {
+                        "type": "serial",
+                        "times": 3,
+                    }
+                },
+                {
+                    "runner": {
+                        "type": "serial",
+                        "times": 10,
+                    }
+                }
+            ]
+        }
+        rally = utils.Rally()
+        task, uuid = self._start_task_in_new_thread(
+            rally, cfg, "test_abort_soft-thread_with_soft_abort.txt")
+        rally("task abort --soft")
+        task.join()
+        results = json.loads(rally("task results"))
+        iterations_completed = len(results[0]["result"])
+        # NOTE(msdubov): check that the task is stopped after first runner
+        #                benchmark finished all its iterations
+        self.assertEqual(3, iterations_completed)
+        # NOTE(msdubov): check that the next benchmark scenario is not started
+        self.assertEqual(1, len(results))
+        self.assertIn("aborted", rally("task status"))
 
     def test_use(self):
         rally = utils.Rally()
-        with mock.patch.dict("os.environ", utils.TEST_ENV):
-            deployment_id = envutils.get_global("RALLY_DEPLOYMENT")
-            config = utils.TaskConfig(self._get_sample_task_config())
-            output = rally(("task start --task %(task_file)s "
-                            "--deployment %(deployment_id)s") %
-                           {"task_file": config.filename,
-                            "deployment_id": deployment_id})
-            result = re.search(
-                r"(?P<uuid>[0-9a-f\-]{36}): started", output)
-            uuid = result.group("uuid")
-            rally("task use --task %s" % uuid)
-            current_task = envutils.get_global("RALLY_TASK")
-            self.assertEqual(uuid, current_task)
+        deployment_id = utils.get_global("RALLY_DEPLOYMENT", rally.env)
+        config = utils.TaskConfig(self._get_sample_task_config())
+        output = rally(("task start --task %(task_file)s "
+                        "--deployment %(deployment_id)s") %
+                       {"task_file": config.filename,
+                        "deployment_id": deployment_id})
+        result = re.search(
+            r"(?P<uuid>[0-9a-f\-]{36}): started", output)
+        uuid = result.group("uuid")
+        rally("task use --task %s" % uuid)
+        current_task = utils.get_global("RALLY_TASK", rally.env)
+        self.assertEqual(uuid, current_task)
 
 
 class SLATestCase(unittest.TestCase):
@@ -556,7 +672,7 @@ class SLATestCase(unittest.TestCase):
         cfg = self._get_sample_task_config(max_seconds_per_iteration=0.001)
         config = utils.TaskConfig(cfg)
         rally("task start --task %s" % config.filename)
-        self.assertRaises(utils.RallyCmdError, rally, "task sla_check")
+        self.assertRaises(utils.RallyCliError, rally, "task sla_check")
 
     def test_sla_success(self):
         rally = utils.Rally()
@@ -609,7 +725,7 @@ class SLAExtraFlagsTestCase(unittest.TestCase):
         ]
         try:
             rally("task sla_check --json", getjson=True)
-        except utils.RallyCmdError as expected_error:
+        except utils.RallyCliError as expected_error:
             self.assertEqual(json.loads(expected_error.output), expected)
         else:
             self.fail("`rally task sla_check` command should return non-zero "
@@ -637,7 +753,7 @@ class SLAExtraFlagsTestCase(unittest.TestCase):
         ]
         try:
             rally("task sla_check --json", getjson=True)
-        except utils.RallyCmdError as expected_error:
+        except utils.RallyCliError as expected_error:
             self.assertEqual(json.loads(expected_error.output), expected)
         else:
             self.fail("`rally task sla_check` command should return non-zero "
