@@ -18,14 +18,36 @@ from keystoneclient import exceptions as keystone_exceptions
 import mock
 from oslo_config import cfg
 
+from rally.common import objects
 from rally import consts
 from rally import exceptions
-from rally import objects
 from rally import osclients
 from tests.unit import fakes
 from tests.unit import test
 
 
+class CachedTestCase(test.TestCase):
+
+    def test_cached(self):
+        foo_client = mock.Mock(
+            __name__="foo_client",
+            side_effect=lambda ins, *args, **kw: (args, kw))
+        ins = mock.Mock(cache={})
+        cached = osclients.cached(foo_client)
+        self.assertEqual(((), {}), cached(ins))
+        self.assertEqual({"foo_client": ((), {})}, ins.cache)
+        self.assertEqual((("foo",), {"bar": "spam"}),
+                         cached(ins, "foo", bar="spam"))
+        self.assertEqual(
+            {"foo_client": ((), {}),
+             "foo_client('foo',){'bar': 'spam'}": (("foo",),
+                                                   {"bar": "spam"})},
+            ins.cache)
+        ins.cache["foo_client('foo',){'bar': 'spam'}"] = "foo_cached"
+        self.assertEqual(
+            "foo_cached", cached(ins, "foo", bar="spam"))
+
+
 class TestCreateKeystoneClient(test.TestCase):
 
     def setUp(self):
@@ -125,23 +147,25 @@ class OSClientsTestCase(test.TestCase):
         self.assertEqual(self.fake_keystone, self.clients.cache["keystone"])
 
     @mock.patch("rally.osclients.Clients.keystone")
-    def test_verified_keystone_user_not_admin(self, mock_keystone):
-        mock_keystone.return_value = fakes.FakeKeystoneClient()
-        mock_keystone.return_value.auth_ref.role_names = ["notadmin"]
+    def test_verified_keystone_user_not_admin(self, mock_clients_keystone):
+        mock_clients_keystone.return_value = fakes.FakeKeystoneClient()
+        mock_clients_keystone.return_value.auth_ref.role_names = ["notadmin"]
         self.assertRaises(exceptions.InvalidAdminException,
                           self.clients.verified_keystone)
 
     @mock.patch("rally.osclients.Clients.keystone")
-    def test_verified_keystone_unauthorized(self, mock_keystone):
-        mock_keystone.return_value = fakes.FakeKeystoneClient()
-        mock_keystone.side_effect = keystone_exceptions.Unauthorized
+    def test_verified_keystone_unauthorized(self, mock_clients_keystone):
+        mock_clients_keystone.return_value = fakes.FakeKeystoneClient()
+        mock_clients_keystone.side_effect = keystone_exceptions.Unauthorized
         self.assertRaises(exceptions.InvalidEndpointsException,
                           self.clients.verified_keystone)
 
     @mock.patch("rally.osclients.Clients.keystone")
-    def test_verified_keystone_unreachable(self, mock_keystone):
-        mock_keystone.return_value = fakes.FakeKeystoneClient()
-        mock_keystone.side_effect = keystone_exceptions.AuthorizationFailure
+    def test_verified_keystone_unreachable(self, mock_clients_keystone):
+        mock_clients_keystone.return_value = fakes.FakeKeystoneClient()
+        mock_clients_keystone.side_effect = (
+            keystone_exceptions.AuthorizationFailure
+        )
         self.assertRaises(exceptions.HostUnreachableException,
                           self.clients.verified_keystone)
 
@@ -244,6 +268,36 @@ class OSClientsTestCase(test.TestCase):
                              self.fake_keystone.auth_token)
             self.assertEqual(fake_cinder, self.clients.cache["cinder"])
 
+    def test_manila(self):
+        mock_manila = mock.MagicMock()
+        self.assertNotIn("manila", self.clients.cache)
+        with mock.patch.dict("sys.modules", {"manilaclient": mock_manila}):
+            client = self.clients.manila()
+            self.assertEqual(mock_manila.client.Client.return_value, client)
+            self.service_catalog.url_for.assert_called_once_with(
+                service_type="share",
+                endpoint_type=consts.EndpointType.PUBLIC,
+                region_name=self.endpoint.region_name)
+            mock_manila.client.Client.assert_called_once_with(
+                "1",
+                http_log_debug=False,
+                timeout=cfg.CONF.openstack_client_http_timeout,
+                insecure=False, cacert=None,
+                username=self.endpoint.username,
+                api_key=self.endpoint.password,
+                region_name=self.endpoint.region_name,
+                project_name=self.endpoint.tenant_name,
+                auth_url=self.endpoint.auth_url)
+            self.assertEqual(
+                mock_manila.client.Client.return_value.client.management_url,
+                self.service_catalog.url_for.return_value)
+            self.assertEqual(
+                mock_manila.client.Client.return_value.client.auth_token,
+                self.fake_keystone.auth_token)
+            self.assertEqual(
+                mock_manila.client.Client.return_value,
+                self.clients.cache["manila"])
+
     def test_ceilometer(self):
         fake_ceilometer = fakes.FakeCeilometerClient()
         mock_ceilometer = mock.MagicMock()
@@ -292,7 +346,7 @@ class OSClientsTestCase(test.TestCase):
                 "insecure": self.endpoint.insecure,
                 "cacert": self.endpoint.cacert
             }
-            mock_ironic.client.get_client.assert_called_once_with("1.0", **kw)
+            mock_ironic.client.get_client.assert_called_once_with("1", **kw)
             self.assertEqual(fake_ironic, self.clients.cache["ironic"])
 
     def test_sahara(self):
@@ -377,9 +431,9 @@ class OSClientsTestCase(test.TestCase):
             )
             fake_mistral_url = self.service_catalog.url_for.return_value
             mock_mistral.client.client.assert_called_once_with(
-                 mistral_url=fake_mistral_url,
-                 service_type="workflowv2",
-                 auth_token=self.fake_keystone.auth_token
+                mistral_url=fake_mistral_url,
+                service_type="workflowv2",
+                auth_token=self.fake_keystone.auth_token
             )
             self.assertEqual(fake_mistral, self.clients.cache["mistral"])
 
@@ -403,7 +457,7 @@ class OSClientsTestCase(test.TestCase):
                   "user": self.endpoint.username,
                   "key": self.endpoint.password,
                   "tenant_name": self.endpoint.tenant_name,
-                  "auth_url": self.endpoint.auth_url
+                  "authurl": self.endpoint.auth_url
                   }
             mock_swift.client.Connection.assert_called_once_with(**kw)
             self.assertEqual(self.clients.cache["swift"], fake_swift)
@@ -435,11 +489,12 @@ class OSClientsTestCase(test.TestCase):
             self.assertEqual(fake_ec2, self.clients.cache["ec2"])
 
     @mock.patch("rally.osclients.Clients.keystone")
-    def test_services(self, mock_keystone):
+    def test_services(self, mock_clients_keystone):
         available_services = {consts.ServiceType.IDENTITY: {},
                               consts.ServiceType.COMPUTE: {},
                               "unknown_service": {}}
-        mock_keystone.return_value = mock.Mock(service_catalog=mock.Mock(
+        mock_clients_keystone.return_value = mock.Mock(
+            service_catalog=mock.Mock(
                 get_endpoints=lambda: available_services))
         clients = osclients.Clients(self.endpoint)
 
@@ -465,3 +520,23 @@ class OSClientsTestCase(test.TestCase):
                   "token": self.fake_keystone.auth_token}
             mock_murano.client.Client.assert_called_once_with("1", **kw)
             self.assertEqual(fake_murano, self.clients.cache["murano"])
+
+    @mock.patch("rally.osclients.cached")
+    def test_register(self, mock_cached):
+        client_func = mock.Mock(return_value="foo_client")
+        cached_client_func = mock.Mock(return_value="cached_foo_client")
+        mock_cached.return_value = cached_client_func
+        clients = osclients.Clients(mock.Mock())
+
+        self.assertFalse(hasattr(clients, "foo"))
+
+        func = osclients.Clients.register("foo")(client_func)
+
+        mock_cached.assert_called_once_with(client_func)
+        self.assertEqual("cached_foo_client", clients.foo())
+        self.assertEqual(client_func, func)
+        self.assertEqual(cached_client_func, clients.foo)
+
+        # Call second time with same name
+        self.assertRaises(ValueError,
+                          osclients.Clients.register("foo"), client_func)
